@@ -1216,7 +1216,7 @@ mod tests {
         let session_ctx = SessionContext::new();
         let store = object_store::memory::InMemory::new();
 
-        let data = bytes::Bytes::from("a,b\n1,2\n3,4");
+        let data = Bytes::from("a,b\n1,2\n3,4");
         let path = object_store::path::Path::from("a.csv");
         store.put(&path, data.into()).await.unwrap();
 
@@ -1247,7 +1247,7 @@ mod tests {
         let session_ctx = SessionContext::new();
         let store = object_store::memory::InMemory::new();
 
-        let data = bytes::Bytes::from("a,b\r1,2\r3,4");
+        let data = Bytes::from("a,b\r1,2\r3,4");
         let path = object_store::path::Path::from("a.csv");
         store.put(&path, data.into()).await.unwrap();
 
