@@ -22,12 +22,9 @@
 //!
 //! [`ExecutionPlan`]: crate::physical_plan::ExecutionPlan
 pub mod coalesce_batches;
-pub mod combine_partial_final_agg;
 pub mod enforce_distribution;
 pub mod enforce_sorting;
 pub mod join_selection;
-pub mod limit_pushdown;
-pub mod limited_distinct_aggregation;
 pub mod optimizer;
 pub mod projection_pushdown;
 pub mod pruning;
@@ -35,7 +32,6 @@ pub mod replace_with_order_preserving_variants;
 pub mod sanity_checker;
 #[cfg(test)]
 pub mod test_utils;
-pub mod topk_aggregation;
 pub mod update_aggr_exprs;
 
 mod sort_pushdown;
