@@ -65,7 +65,7 @@ pub mod select;
 mod sql_api;
 
 async fn register_aggregate_csv_by_sql(ctx: &SessionContext) {
-    let testdata = datafusion::test_util::arrow_test_data();
+    let testdata = test_util::arrow_test_data();
 
     let df = ctx
         .sql(&format!(
@@ -103,7 +103,7 @@ async fn register_aggregate_csv_by_sql(ctx: &SessionContext) {
 }
 
 async fn register_aggregate_csv(ctx: &SessionContext) -> Result<()> {
-    let testdata = datafusion::test_util::arrow_test_data();
+    let testdata = test_util::arrow_test_data();
     let schema = test_util::aggr_test_schema();
     ctx.register_csv(
         "aggregate_test_100",
@@ -227,7 +227,7 @@ fn result_vec(results: &[RecordBatch]) -> Vec<Vec<String>> {
 }
 
 async fn register_alltypes_parquet(ctx: &SessionContext) {
-    let testdata = datafusion::test_util::parquet_test_data();
+    let testdata = test_util::parquet_test_data();
     ctx.register_parquet(
         "alltypes_plain",
         &format!("{testdata}/alltypes_plain.parquet"),
