@@ -78,7 +78,6 @@ pub mod average;
 pub mod bit_and_or_xor;
 pub mod bool_and_or;
 pub mod grouping;
-pub mod kurtosis_pop;
 pub mod nth_value;
 pub mod string_agg;
 
@@ -171,7 +170,6 @@ pub fn all_default_aggregate_functions() -> Vec<Arc<AggregateUDF>> {
         average::avg_udaf(),
         grouping::grouping_udaf(),
         nth_value::nth_value_udaf(),
-        kurtosis_pop::kurtosis_pop_udaf(),
     ]
 }
 
