@@ -15,17 +15,19 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::any::Any;
-
+use crate::datetime::common::*;
 use arrow::datatypes::DataType;
-use arrow::datatypes::DataType::Date32;
+use arrow::datatypes::DataType::*;
 use arrow::error::ArrowError::ParseError;
 use arrow::{array::types::Date32Type, compute::kernels::cast_utils::Parser};
-
-use crate::datetime::common::*;
 use datafusion_common::error::DataFusionError;
 use datafusion_common::{arrow_err, exec_err, internal_datafusion_err, Result};
-use datafusion_expr::{ColumnarValue, ScalarUDFImpl, Signature, Volatility};
+use datafusion_expr::scalar_doc_sections::DOC_SECTION_DATETIME;
+use datafusion_expr::{
+    ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
+};
+use std::any::Any;
+use std::sync::OnceLock;
 
 #[derive(Debug)]
 pub struct ToDateFunc {
@@ -77,6 +79,50 @@ impl ToDateFunc {
     }
 }
 
+static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
+
+fn get_to_date_doc() -> &'static Documentation {
+    DOCUMENTATION.get_or_init(|| {
+        Documentation::builder()
+            .with_doc_section(DOC_SECTION_DATETIME)
+            .with_description(r#"Converts a value to a date (`YYYY-MM-DD`).
+Supports strings, integer and double types as input.
+Strings are parsed as YYYY-MM-DD (e.g. '2023-07-20') if no [Chrono format](https://docs.rs/chrono/latest/chrono/format/strftime/index.html)s are provided.
+Integers and doubles are interpreted as days since the unix epoch (`1970-01-01T00:00:00Z`).
+Returns the corresponding date.
+
+Note: `to_date` returns Date32, which represents its values as the number of days since unix epoch(`1970-01-01`) stored as signed 32 bit value. The largest supported date value is `9999-12-31`.
+"#)
+            .with_syntax_example("to_date('2017-05-31', '%Y-%m-%d')")
+            .with_sql_example(r#"```sql
+> select to_date('2023-01-31');
++-----------------------------+
+| to_date(Utf8("2023-01-31")) |
++-----------------------------+
+| 2023-01-31                  |
++-----------------------------+
+> select to_date('2023/01/31', '%Y-%m-%d', '%Y/%m/%d');
++---------------------------------------------------------------+
+| to_date(Utf8("2023/01/31"),Utf8("%Y-%m-%d"),Utf8("%Y/%m/%d")) |
++---------------------------------------------------------------+
+| 2023-01-31                                                    |
++---------------------------------------------------------------+
+```
+
+Additional examples can be found [here](https://github.com/apache/datafusion/blob/main/datafusion-examples/examples/to_date.rs)
+"#)
+            .with_standard_argument("expression", Some("String"))
+            .with_argument(
+                "format_n",
+                "Optional [Chrono format](https://docs.rs/chrono/latest/chrono/format/strftime/index.html) strings to use to parse the expression. Formats will be tried in the order
+  they appear with the first successful one being returned. If none of the formats successfully parse the expression
+  an error will be returned.",
+            )
+            .build()
+            .unwrap()
+    })
+}
+
 impl ScalarUDFImpl for ToDateFunc {
     fn as_any(&self) -> &dyn Any {
         self
@@ -105,25 +151,28 @@ impl ScalarUDFImpl for ToDateFunc {
         }
 
         match args[0].data_type() {
-            DataType::Int32
-            | DataType::Int64
-            | DataType::Null
-            | DataType::Float64
-            | DataType::Date32
-            | DataType::Date64 => args[0].cast_to(&DataType::Date32, None),
-            DataType::Utf8 => self.to_date(args),
+            Int32 | Int64 | Null | Float64 | Date32 | Date64 => {
+                args[0].cast_to(&Date32, None)
+            }
+            Utf8View | LargeUtf8 | Utf8 => self.to_date(args),
             other => {
                 exec_err!("Unsupported data type {:?} for function to_date", other)
             }
         }
     }
+
+    fn documentation(&self) -> Option<&Documentation> {
+        Some(get_to_date_doc())
+    }
 }
 
 #[cfg(test)]
 mod tests {
+    use arrow::array::{Array, Date32Array, GenericStringArray, StringViewArray};
     use arrow::{compute::kernels::cast_utils::Parser, datatypes::Date32Type};
     use datafusion_common::ScalarValue;
     use datafusion_expr::{ColumnarValue, ScalarUDFImpl};
+    use std::sync::Arc;
 
     use super::ToDateFunc;
 
@@ -154,9 +203,18 @@ mod tests {
         ];
 
         for tc in &test_cases {
-            let date_scalar = ScalarValue::Utf8(Some(tc.date_str.to_string()));
-            let to_date_result =
-                ToDateFunc::new().invoke(&[ColumnarValue::Scalar(date_scalar)]);
+            test_scalar(ScalarValue::Utf8(Some(tc.date_str.to_string())), tc);
+            test_scalar(ScalarValue::LargeUtf8(Some(tc.date_str.to_string())), tc);
+            test_scalar(ScalarValue::Utf8View(Some(tc.date_str.to_string())), tc);
+
+            test_array::<GenericStringArray<i32>>(tc);
+            test_array::<GenericStringArray<i64>>(tc);
+            test_array::<StringViewArray>(tc);
+        }
+
+        fn test_scalar(sv: ScalarValue, tc: &TestCase) {
+            #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
+            let to_date_result = ToDateFunc::new().invoke(&[ColumnarValue::Scalar(sv)]);
 
             match to_date_result {
                 Ok(ColumnarValue::Scalar(ScalarValue::Date32(date_val))) => {
@@ -170,6 +228,34 @@ mod tests {
                 _ => panic!("Could not convert '{}' to Date", tc.date_str),
             }
         }
+
+        fn test_array<A>(tc: &TestCase)
+        where
+            A: From<Vec<&'static str>> + Array + 'static,
+        {
+            let date_array = A::from(vec![tc.date_str]);
+            #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
+            let to_date_result =
+                ToDateFunc::new().invoke(&[ColumnarValue::Array(Arc::new(date_array))]);
+
+            match to_date_result {
+                Ok(ColumnarValue::Array(a)) => {
+                    assert_eq!(a.len(), 1);
+
+                    let expected = Date32Type::parse_formatted(tc.date_str, "%Y-%m-%d");
+                    let mut builder = Date32Array::builder(4);
+                    builder.append_value(expected.unwrap());
+
+                    assert_eq!(
+                        &builder.finish() as &dyn Array,
+                        a.as_ref(),
+                        "{}: to_date created wrong value",
+                        tc.name
+                    );
+                }
+                _ => panic!("Could not convert '{}' to Date", tc.date_str),
+            }
+        }
     }
 
     #[test]
@@ -221,12 +307,27 @@ mod tests {
         ];
 
         for tc in &test_cases {
-            let formatted_date_scalar =
-                ScalarValue::Utf8(Some(tc.formatted_date.to_string()));
+            test_scalar(ScalarValue::Utf8(Some(tc.formatted_date.to_string())), tc);
+            test_scalar(
+                ScalarValue::LargeUtf8(Some(tc.formatted_date.to_string())),
+                tc,
+            );
+            test_scalar(
+                ScalarValue::Utf8View(Some(tc.formatted_date.to_string())),
+                tc,
+            );
+
+            test_array::<GenericStringArray<i32>>(tc);
+            test_array::<GenericStringArray<i64>>(tc);
+            test_array::<StringViewArray>(tc);
+        }
+
+        fn test_scalar(sv: ScalarValue, tc: &TestCase) {
             let format_scalar = ScalarValue::Utf8(Some(tc.format_str.to_string()));
 
+            #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
             let to_date_result = ToDateFunc::new().invoke(&[
-                ColumnarValue::Scalar(formatted_date_scalar),
+                ColumnarValue::Scalar(sv),
                 ColumnarValue::Scalar(format_scalar),
             ]);
 
@@ -241,6 +342,42 @@ mod tests {
                 ),
             }
         }
+
+        fn test_array<A>(tc: &TestCase)
+        where
+            A: From<Vec<&'static str>> + Array + 'static,
+        {
+            let date_array = A::from(vec![tc.formatted_date]);
+            let format_array = A::from(vec![tc.format_str]);
+
+            #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
+            let to_date_result = ToDateFunc::new().invoke(&[
+                ColumnarValue::Array(Arc::new(date_array)),
+                ColumnarValue::Array(Arc::new(format_array)),
+            ]);
+
+            match to_date_result {
+                Ok(ColumnarValue::Array(a)) => {
+                    assert_eq!(a.len(), 1);
+
+                    let expected = Date32Type::parse_formatted(tc.date_str, "%Y-%m-%d");
+                    let mut builder = Date32Array::builder(4);
+                    builder.append_value(expected.unwrap());
+
+                    assert_eq!(
+                        &builder.finish() as &dyn Array, a.as_ref(),
+                        "{}: to_date created wrong value for date '{}' with format string '{}'",
+                        tc.name,
+                        tc.formatted_date,
+                        tc.format_str
+                    );
+                }
+                _ => panic!(
+                    "Could not convert '{}' with format string '{}'to Date: {:?}",
+                    tc.formatted_date, tc.format_str, to_date_result
+                ),
+            }
+        }
     }
 
     #[test]
@@ -249,6 +386,7 @@ mod tests {
         let format1_scalar = ScalarValue::Utf8(Some("%Y-%m-%d".into()));
         let format2_scalar = ScalarValue::Utf8(Some("%Y/%m/%d".into()));
 
+        #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
         let to_date_result = ToDateFunc::new().invoke(&[
             ColumnarValue::Scalar(formatted_date_scalar),
             ColumnarValue::Scalar(format1_scalar),
@@ -277,6 +415,7 @@ mod tests {
         for date_str in test_cases {
             let formatted_date_scalar = ScalarValue::Utf8(Some(date_str.into()));
 
+            #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
             let to_date_result =
                 ToDateFunc::new().invoke(&[ColumnarValue::Scalar(formatted_date_scalar)]);
 
@@ -295,6 +434,7 @@ mod tests {
         let date_str = "20241231";
         let date_scalar = ScalarValue::Utf8(Some(date_str.into()));
 
+        #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
         let to_date_result =
             ToDateFunc::new().invoke(&[ColumnarValue::Scalar(date_scalar)]);
 
@@ -316,6 +456,7 @@ mod tests {
         let date_str = "202412311";
         let date_scalar = ScalarValue::Utf8(Some(date_str.into()));
 
+        #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
         let to_date_result =
             ToDateFunc::new().invoke(&[ColumnarValue::Scalar(date_scalar)]);
 
