@@ -92,9 +92,6 @@ pub mod macros;
 pub mod string;
 make_stub_package!(string, "string_expressions");
 
-#[cfg(feature = "string_expressions")]
-mod regexp_common;
-
 /// Core datafusion expressions
 /// Enabled via feature flag `core_expressions`
 #[cfg(feature = "core_expressions")]
@@ -138,6 +135,8 @@ make_stub_package!(unicode, "unicode_expressions");
 #[cfg(any(feature = "datetime_expressions", feature = "unicode_expressions"))]
 pub mod planner;
 
+pub mod strings;
+
 mod utils;
 
 /// Fluent-style API for creating `Expr`s
