@@ -16,7 +16,7 @@
 // under the License.
 
 use std::any::Any;
-use std::sync::Arc;
+use std::sync::{Arc, OnceLock};
 
 use arrow::array::{ArrayRef, Int64Array};
 use arrow::datatypes::DataType;
@@ -24,8 +24,10 @@ use arrow::datatypes::DataType::Int64;
 
 use arrow::error::ArrowError;
 use datafusion_common::{arrow_datafusion_err, exec_err, DataFusionError, Result};
-use datafusion_expr::ColumnarValue;
-use datafusion_expr::{ScalarUDFImpl, Signature, Volatility};
+use datafusion_expr::scalar_doc_sections::DOC_SECTION_MATH;
+use datafusion_expr::{
+    ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
+};
 
 use super::gcd::unsigned_gcd;
 use crate::utils::make_scalar_function;
@@ -70,6 +72,27 @@ impl ScalarUDFImpl for LcmFunc {
     fn invoke(&self, args: &[ColumnarValue]) -> Result<ColumnarValue> {
         make_scalar_function(lcm, vec![])(args)
     }
+
+    fn documentation(&self) -> Option<&Documentation> {
+        Some(get_lcm_doc())
+    }
+}
+
+static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
+
+fn get_lcm_doc() -> &'static Documentation {
+    DOCUMENTATION.get_or_init(|| {
+        Documentation::builder()
+            .with_doc_section(DOC_SECTION_MATH)
+            .with_description(
+                "Returns the least common multiple of `expression_x` and `expression_y`. Returns 0 if either input is zero.",
+            )
+            .with_syntax_example("lcm(expression_x, expression_y)")
+            .with_standard_argument("expression_x", Some("First numeric"))
+            .with_standard_argument("expression_y", Some("Second numeric"))
+            .build()
+            .unwrap()
+    })
 }
 
 /// Lcm SQL function
