@@ -16,12 +16,16 @@
 // under the License.
 
 use std::any::Any;
+use std::sync::OnceLock;
 
 use arrow::datatypes::DataType;
 use arrow::datatypes::DataType::Float64;
 use datafusion_common::{not_impl_err, Result, ScalarValue};
+use datafusion_expr::scalar_doc_sections::DOC_SECTION_MATH;
 use datafusion_expr::sort_properties::{ExprProperties, SortProperties};
-use datafusion_expr::{ColumnarValue, ScalarUDFImpl, Signature, Volatility};
+use datafusion_expr::{
+    ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
+};
 
 #[derive(Debug)]
 pub struct PiFunc {
@@ -73,4 +77,21 @@ impl ScalarUDFImpl for PiFunc {
         // This function returns a constant value.
         Ok(SortProperties::Singleton)
     }
+
+    fn documentation(&self) -> Option<&Documentation> {
+        Some(get_pi_doc())
+    }
+}
+
+static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
+
+fn get_pi_doc() -> &'static Documentation {
+    DOCUMENTATION.get_or_init(|| {
+        Documentation::builder()
+            .with_doc_section(DOC_SECTION_MATH)
+            .with_description("Returns an approximate value of π.")
+            .with_syntax_example("pi()")
+            .build()
+            .unwrap()
+    })
 }
