@@ -15,16 +15,16 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::any::Any;
-
 use arrow::compute::kernels::length::length;
 use arrow::datatypes::DataType;
-
-use datafusion_common::{exec_err, Result, ScalarValue};
-use datafusion_expr::{ColumnarValue, Volatility};
-use datafusion_expr::{ScalarUDFImpl, Signature};
+use std::any::Any;
+use std::sync::OnceLock;
 
 use crate::utils::utf8_to_int_type;
+use datafusion_common::{exec_err, Result, ScalarValue};
+use datafusion_expr::scalar_doc_sections::DOC_SECTION_STRING;
+use datafusion_expr::{ColumnarValue, Documentation, Volatility};
+use datafusion_expr::{ScalarUDFImpl, Signature};
 
 #[derive(Debug)]
 pub struct OctetLengthFunc {
@@ -39,13 +39,8 @@ impl Default for OctetLengthFunc {
 
 impl OctetLengthFunc {
     pub fn new() -> Self {
-        use DataType::*;
         Self {
-            signature: Signature::uniform(
-                1,
-                vec![Utf8, LargeUtf8, Utf8View],
-                Volatility::Immutable,
-            ),
+            signature: Signature::string(1, Volatility::Immutable),
         }
     }
 }
@@ -91,6 +86,36 @@ impl ScalarUDFImpl for OctetLengthFunc {
             },
         }
     }
+
+    fn documentation(&self) -> Option<&Documentation> {
+        Some(get_octet_length_doc())
+    }
+}
+
+static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
+
+fn get_octet_length_doc() -> &'static Documentation {
+    DOCUMENTATION.get_or_init(|| {
+        Documentation::builder()
+            .with_doc_section(DOC_SECTION_STRING)
+            .with_description("Returns the length of a string in bytes.")
+            .with_syntax_example("octet_length(str)")
+            .with_sql_example(
+                r#"```sql
+> select octet_length('Ångström');
++--------------------------------+
+| octet_length(Utf8("Ångström")) |
++--------------------------------+
+| 10                             |
++--------------------------------+
+```"#,
+            )
+            .with_standard_argument("str", Some("String"))
+            .with_related_udf("bit_length")
+            .with_related_udf("length")
+            .build()
+            .unwrap()
+    })
 }
 
 #[cfg(test)]
