@@ -17,7 +17,7 @@
 
 use std::any::Any;
 use std::fmt::Write;
-use std::sync::Arc;
+use std::sync::{Arc, OnceLock};
 
 use arrow::array::{
     Array, ArrayRef, AsArray, GenericStringArray, GenericStringBuilder, Int64Array,
@@ -27,13 +27,15 @@ use arrow::datatypes::DataType;
 use unicode_segmentation::UnicodeSegmentation;
 use DataType::{LargeUtf8, Utf8, Utf8View};
 
+use crate::strings::StringArrayType;
+use crate::utils::{make_scalar_function, utf8_to_str_type};
 use datafusion_common::cast::as_int64_array;
 use datafusion_common::{exec_err, Result};
+use datafusion_expr::scalar_doc_sections::DOC_SECTION_STRING;
 use datafusion_expr::TypeSignature::Exact;
-use datafusion_expr::{ColumnarValue, ScalarUDFImpl, Signature, Volatility};
-
-use crate::string::common::StringArrayType;
-use crate::utils::{make_scalar_function, utf8_to_str_type};
+use datafusion_expr::{
+    ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
+};
 
 #[derive(Debug)]
 pub struct LPadFunc {
@@ -95,6 +97,35 @@ impl ScalarUDFImpl for LPadFunc {
             other => exec_err!("Unsupported data type {other:?} for function lpad"),
         }
     }
+
+    fn documentation(&self) -> Option<&Documentation> {
+        Some(get_lpad_doc())
+    }
+}
+
+static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
+
+fn get_lpad_doc() -> &'static Documentation {
+    DOCUMENTATION.get_or_init(|| {
+        Documentation::builder()
+            .with_doc_section(DOC_SECTION_STRING)
+            .with_description("Pads the left side of a string with another string to a specified string length.")
+            .with_syntax_example("lpad(str, n[, padding_str])")
+            .with_sql_example(r#"```sql
+> select lpad('Dolly', 10, 'hello');
++---------------------------------------------+
+| lpad(Utf8("Dolly"),Int64(10),Utf8("hello")) |
++---------------------------------------------+
+| helloDolly                                  |
++---------------------------------------------+
+```"#)
+            .with_standard_argument("str", Some("String"))
+            .with_argument("n", "String length to pad to.")
+            .with_argument("padding_str", "Optional string expression to pad with. Can be a constant, column, or function, and any combination of string operators. _Default is a space._")
+            .with_related_udf("rpad")
+            .build()
+            .unwrap()
+    })
 }
 
 /// Extends the string to length 'length' by prepending the characters fill (a space by default).
