@@ -17,7 +17,7 @@
 
 use std::any::Any;
 use std::cmp::{max, Ordering};
-use std::sync::Arc;
+use std::sync::{Arc, OnceLock};
 
 use arrow::array::{
     Array, ArrayAccessor, ArrayIter, ArrayRef, GenericStringArray, Int64Array,
@@ -31,8 +31,11 @@ use datafusion_common::cast::{
 };
 use datafusion_common::exec_err;
 use datafusion_common::Result;
+use datafusion_expr::scalar_doc_sections::DOC_SECTION_STRING;
 use datafusion_expr::TypeSignature::Exact;
-use datafusion_expr::{ColumnarValue, ScalarUDFImpl, Signature, Volatility};
+use datafusion_expr::{
+    ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
+};
 
 #[derive(Debug)]
 pub struct RightFunc {
@@ -90,6 +93,34 @@ impl ScalarUDFImpl for RightFunc {
             ),
         }
     }
+
+    fn documentation(&self) -> Option<&Documentation> {
+        Some(get_right_doc())
+    }
+}
+
+static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
+
+fn get_right_doc() -> &'static Documentation {
+    DOCUMENTATION.get_or_init(|| {
+        Documentation::builder()
+            .with_doc_section(DOC_SECTION_STRING)
+            .with_description("Returns a specified number of characters from the right side of a string.")
+            .with_syntax_example("right(str, n)")
+            .with_sql_example(r#"```sql
+> select right('datafusion', 6);
++------------------------------------+
+| right(Utf8("datafusion"),Int64(6)) |
++------------------------------------+
+| fusion                             |
++------------------------------------+
+```"#)
+            .with_standard_argument("str", Some("String"))
+            .with_argument("n", "Number of characters to return")
+            .with_related_udf("left")
+            .build()
+            .unwrap()
+    })
 }
 
 /// Returns last n characters in the string, or when n is negative, returns all but first |n| characters.
