@@ -18,11 +18,7 @@
 mod aggregate;
 mod built_in;
 mod built_in_window_function_expr;
-pub(crate) mod cume_dist;
-pub(crate) mod lead_lag;
 pub(crate) mod nth_value;
-pub(crate) mod ntile;
-pub(crate) mod rank;
 mod sliding_aggregate;
 mod window_expr;
 
